@echo off

setlocal

rem Default name for new mods (Optional)
set AUTHOR=
rem Script uses its location as mod directory
set MODDIR=%~dp0
set WORKDIR=%MODDIR%\worktree

if "%~1"=="" (
	echo Factorio mod worktree script.
	echo -----
	echo   1. Initialize new mod
	echo   2. Update versions
	echo   0. Exit
	echo -----

	choice /C 012 /N /M "Select an option by pressing digit key: "
	if ERRORLEVEL 3 (
		goto updateMods
	) else if ERRORLEVEL 2 (
		goto createMod
	)
) else if "%~1"=="create" (
	goto createMod
) else if "%~1"=="update" (
	goto updateMods
) else (
	echo Invalid mode specified
)

endlocal

exit /B

:createMod
if NOT "%~2"=="" (set MODNAME=%~2) else set /P MODNAME=Enter mod name: 
if "%MODNAME%"=="" (
	echo Mod name must not be empty.
	exit /B
)
if EXIST "%WORKDIR%\%MODNAME%" (
	echo Mod with such name already exists, choose a different name or remove existing mod from work tree.
	exit /B
)
if NOT "%~3"=="" (set MODTITLE=%~3) else set /P MODTITLE=Enter mod title[%MODNAME%]: 
if "%MODTITLE%"=="" set MODTITLE=%MODNAME%
if NOT "%~4"=="" (
	set MODAUTHOR=%~4
) else if NOT "%AUTHOR%"=="" (
	set MODAUTHOR=%AUTHOR%
) else set /P MODAUTHOR=Enter mod author[default]: 
if "%MODAUTHOR%"=="" set MODAUTHOR=default

mkdir "%WORKDIR%\%MODNAME%"
mklink /D "%MODDIR%\%MODNAME%_0.0.0" "%WORKDIR%\%MODNAME%"

set IJS="%WORKDIR%\%MODNAME%\info.json"
echo { > %IJS%
echo   "name": "%MODNAME%", >> %IJS%
echo   "version": "0.0.0", >> %IJS%
echo   "title": "%MODTITLE%", >> %IJS%
echo   "author": "%MODAUTHOR%" >> %IJS%
echo } >> %IJS%

echo Mod directory initialized at %WORKDIR%\%MODNAME%
echo To sync mod version run "%~n0 update" or launch script in interactive mode and choose "Update versions".

exit /B

:updateMods
setlocal EnableDelayedExpansion
echo Updating mods versions
for /F "tokens=*" %%A in ('dir /B /A:D %WORKDIR%') do (
	if EXIST "%WORKDIR%\%%~A\info.json" (
		call :readVersion MODVER "%WORKDIR%\%%~A\info.json"
		call :findModLink MODLINK "%MODDIR%\%%~A"
		if NOT "!MODLINK!"=="" (
			if NOT "%MODDIR%\!MODLINK!"=="%MODDIR%\%%~A_!MODVER!" (
				move "%MODDIR%\!MODLINK!" "%MODDIR%\%%~A_!MODVER!" > NUL
				echo Mod %%A[!MODVER!] link updated
			) else (
				echo Skipping %%A, up to date
			)
		) else (
			mklink /D "%MODDIR%\%%~A_!MODVER!" "%WORKDIR%\%%~A"
			echo Mod %%A[!MODVER!] link created
		)
	)
)
endlocal
exit /B

:findModLink
for /F "tokens=*" %%A in ('dir /B /A:L "%~2_*"') do (
	set %~1=%%A
	exit /B
)
exit /B

:readVersion
for /F "tokens=2 delims=:" %%B in ('findstr /R /C:"\"version\"" %2') do (
	set VER=%%B
	set VER=!VER:,=!
	call :trimSpaceAndQuotes VER !VER!
	set %~1=!VER:,=!
)
exit /B

:trimSpaceAndQuotes
set PARAMS=%*
for /F "tokens=1*" %%C in ("%PARAMS%") do set %~1=%%~D
exit /B